use crate::Mem;
use minifb::{Window, WindowOptions};
use std::{
    sync::{Arc, RwLock},
    thread::sleep,
    time::{Duration, Instant},
};

pub struct Crtc {
    memory: Arc<RwLock<Mem>>,
    buffer: Vec<u32>,
    window: minifb::Window,
}

impl Crtc {
    pub fn new(memory: Arc<RwLock<Mem>>) -> Self {
        let window = Window::new("screen", 512, 380, WindowOptions::default()).unwrap();
        Self {
            memory,
            buffer: vec![0; 512 * 380],
            window,
        }
    }
    fn draw(&mut self) {
        let memory = match self.memory.try_read() {
            Ok(read) => read,
            Err(_) => {
                println!("Couldn't acquire read on shared memory in main thread");
                return;
            }
        };
        // let bitmask = [128, 64, 32, 16, 8, 4, 2, 1];
        let bitmask = [1, 2, 4, 8, 16, 32, 64, 128]; // I think this is how the bytes will be
                                                     // serialized? low bits first?

        for addr in 0x6000..0xBF00 {
            // TODO: eventually this will access memory in the weird interleaved way the hardware is
            // designed, but this is easiest to implement for now
            let byte = memory.read(addr).unwrap();
            for (i, mask) in bitmask.iter().enumerate() {
                match byte & mask == 0 {
                    true => self.buffer[(addr - 0x6000) as usize * 8 + i] = 0x110500,
                    false => self.buffer[(addr - 0x6000) as usize * 8 + i] = 0xFFCC00,
                }
            }
        }
        self.window
            .update_with_buffer(&self.buffer, 512, 380)
            .unwrap();
    }
    pub fn run(&mut self) {
        let frame_duration = Duration::from_millis(16);
        let mut previous_draw = Instant::now();

        loop {
            let now = Instant::now();
            if now - previous_draw > frame_duration {
                self.draw();
                previous_draw = now;
            }
            sleep(Duration::from_millis(1));
        }
    }
}
